Package: iscream
Title: Make fast and memory efficient BED file queries, summaries and
        matrices
Version: 1.0.0
Authors@R: c(
    person("James", "Eapen", email = "james.eapen@vai.org", role = c("aut", "cre"), comment = c(ORCID = "0000-0001-6016-3598")),
    person("Jacob", "Morrison", email = "jacob.morrison@vai.org", role = c("aut"), comment = c(ORCID = "0000-0001-8592-4744")),
    person("Nathan", "Spix", role = c("ctb")),
    person("Hui", "Shen", email = "hui.shen@vai.org", role = c("aut", "ths", "fnd"), comment = c(ORCID = "0000-0001-9767-4084")))
Description: BED files store ranged genomic data that can be queried even when
             the files are compressed. iscream can query data from BED files and return them
             in muliple formats: parsed records or their summary statistics as data frames
             or GenomicRanges objects, and matrices as matrix, GenomicRanges, or
             SummarizedExperiment objects. iscream also provides specialized
             support for importing methylation data.
URL: https://huishenlab.github.io/iscream/,
        https://github.com/huishenlab/iscream/
BugReports: https://github.com/huishenlab/iscream/issues/
License: MIT + file LICENSE
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.3
Depends: R (>= 4.5)
LinkingTo: Rcpp, RcppArmadillo, RcppProgress, RcppSpdlog, Rhtslib,
        stringfish
Imports: Rcpp, Matrix, data.table, methods, pbapply, parallelly,
        stringfish,
Suggests: BiocFileCache, BiocStyle, bsseq, ggplot2, ggridges, knitr,
        microbenchmark, rmarkdown, GenomicRanges, IRanges, Rsamtools,
        SummarizedExperiment, S4Vectors, testthat (>= 3.0.0)
VignetteBuilder: knitr
Config/testthat/edition: 3
NeedsCompilation: yes
SystemRequirements: htslib: htslib-devel (rpm) or libhts-dev (deb) &
        tabix: htslib-tools (rpm) or tabix (deb) & GNU make
biocViews: DataImport, Software, Sequencing, SingleCell, DNAMethylation
git_url: https://git.bioconductor.org/packages/iscream
git_branch: RELEASE_3_22
git_last_commit: faf115f
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
Packaged: 2025-10-30 04:39:58 UTC; biocbuild
Author: James Eapen [aut, cre] (ORCID: <https://orcid.org/0000-0001-6016-3598>),
  Jacob Morrison [aut] (ORCID: <https://orcid.org/0000-0001-8592-4744>),
  Nathan Spix [ctb],
  Hui Shen [aut, ths, fnd] (ORCID:
    <https://orcid.org/0000-0001-9767-4084>)
Maintainer: James Eapen <james.eapen@vai.org>
