# Generated by roxygen2: do not edit by hand

export(get_df_string)
export(get_granges_string)
export(get_log_level)
export(get_threads)
export(htslib_version)
export(make_mat)
export(make_mat_bsseq)
export(make_mat_gr)
export(make_mat_se)
export(query_chroms)
export(set_log_level)
export(set_threads)
export(summarize_meth_regions)
export(summarize_regions)
export(tabix)
export(tabix_gr)
export(tabix_raw)
import(Matrix)
import(stringfish)
importFrom(Matrix,drop0)
importFrom(Rcpp,sourceCpp)
importFrom(data.table,":=")
importFrom(data.table,as.data.table)
importFrom(data.table,fread)
importFrom(data.table,fwrite)
importFrom(data.table,rbindlist)
importFrom(data.table,set)
importFrom(data.table,setDT)
importFrom(data.table,setnames)
importFrom(data.table,tstrsplit)
importFrom(methods,is)
importFrom(parallelly,availableCores)
importFrom(pbapply,pblapply)
importFrom(pbapply,pboptions)
importFrom(stats,setNames)
importFrom(stringfish,sf_grepl)
importFrom(tools,file_path_sans_ext)
useDynLib(iscream, .registration = TRUE)
