## usethis namespace: start
#' @author James Eapen
#' @name iscream
#' @useDynLib iscream, .registration = TRUE
#' @importFrom Rcpp sourceCpp
#' @import Matrix
#' @import stringfish
## usethis namespace: end
NULL
